/******************************************************************************
 * 
 * Copyright 2012 - 2013 Vektor Software.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 *****************************************************************************/

package com.vektorsoft.demux.android.main;

import android.app.Activity;
import android.os.Bundle;
import android.util.Log;
import com.vektorsoft.demux.android.main.svc.DMXMainActivityService;
import com.vektorsoft.demux.android.main.svc.DMXMainActivityServiceImpl;
import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.apache.felix.framework.Felix;
import org.apache.felix.framework.util.FelixConstants;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleException;
import org.osgi.framework.Constants;
import org.osgi.framework.launch.Framework;

/**
 * Main {@link Activity} class for DEMUX Framework Android applications. This is
 * used as main application window and parent of all sub-views.
 *
 * @author Vladimir Djurovic
 */
public class DMXMainActivity extends Activity {

    /**
     * Logging tag.
     */
    private static final String LOG_TAG = "DMXMainActivity";
    /**
     * Name of bundle cache directory.
     */
    private static final String BUNDLE_CACHE_DIR = "bundle-cache";
    /**
     * Name of the resource containing extra packages list.
     */
    private static final String EXTRA_PACKAGES_RESOURCE = "/packages_ext.txt";
    /**
     * Name of bundle storage directory under assets hierarchy.
     */
    private static final String ASSET_BUNDLE_DIR = "bundles";
    
    /** Amount of time to wait for OSGi framework shutdown (in ms). */
    private static final int WAIT_TIMEOUT = 100;
    
    /**
     * Instance of running OSGI framework.
     */
    private Framework framework;

    @Override
    protected final void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        // find current file system location
        String location = getFilesDir().getAbsolutePath();
        Log.d(LOG_TAG, "Current app location: " + location);

        // create bundle cache directory
        File cacheDir = new File(location, BUNDLE_CACHE_DIR);
        if (!cacheDir.exists()) {
            boolean created = cacheDir.mkdir();
            if (!created) {
                throw new IllegalStateException("Could not create cache directory");
            }
        }

        // create OSGI runtime configuration parameters
        Map<String, Object> config = new HashMap<String, Object>();
        config.put(Constants.FRAMEWORK_STORAGE, cacheDir.getAbsolutePath());
        // add Android packages to extra system packages
        StringBuilder sb = new StringBuilder("");
        InputStream in = getClass().getResourceAsStream(EXTRA_PACKAGES_RESOURCE);

        String line = null;
        if (in != null) {
            BufferedReader reader = new BufferedReader(new InputStreamReader(in, Charset.forName("UTF-8")));
            try {
                while ((line = reader.readLine()) != null) {
                    sb.append(line).append("\n");
                }
            } catch (IOException ex) {
                Log.e(LOG_TAG, "Could not read extra system packages", ex);
            } finally {
                try {
                    reader.close();
                } catch (IOException ex) {
                    Log.e(LOG_TAG, "Could not close reader", ex);
                }
            }
        }

        config.put(Constants.FRAMEWORK_SYSTEMPACKAGES_EXTRA, sb.toString());
        Log.d(LOG_TAG, "Configuration: " + config.toString());
        
        // create activator for this application
        List<BundleActivator> activators = new ArrayList<BundleActivator>();
        
        DMXMainActivityService activityService = new DMXMainActivityServiceImpl(this);
        AndroidMainActivator activator = new AndroidMainActivator(activityService);
        activators.add(activator);
        config.put(FelixConstants.SYSTEMBUNDLE_ACTIVATORS_PROP, activators);

        //create and start framework
        framework = new Felix(config);

        try {
            framework.start();
        } catch (BundleException ex) {
            throw new IllegalStateException("Could not start framework: " + ex.getMessage());
        }

        // get list of bundles from assets
        String[] assetBundles = new String[0];
        try {
            assetBundles = getAssets().list(ASSET_BUNDLE_DIR);
        } catch (IOException ex) {
            Log.e(LOG_TAG, "Could not list asset bundles: " + ex.getMessage());
        }
        // install bundles from assets
        InputStream is = null;
        for (String asset : assetBundles) {
            Log.d(LOG_TAG, "Installing bundle from asset " + asset);
            try {
                is = getAssets().open(ASSET_BUNDLE_DIR + "/" + asset);
                framework.getBundleContext().installBundle(asset, is);
            } catch (IOException ex) {
                Log.e(LOG_TAG, "IOException occured: " + ex.getMessage());
            } catch (BundleException bex) {
                Log.e(LOG_TAG, "Could not install bundle: " + bex.getMessage());
            } finally {
                try {
                    if (is != null) {
                        is.close();
                    }
                } catch (IOException iex) {
                    Log.e(LOG_TAG, "Could not close input stream");
                }
            }

        }

        Log.d(LOG_TAG, "Number of installed bundles: " + framework.getBundleContext().getBundles().length);
        // start all installed bundles
        for (org.osgi.framework.Bundle bundle : framework.getBundleContext().getBundles()) {
            Log.d(LOG_TAG, "Trying to start bundle " + bundle.getSymbolicName());
            try {
                // if it is frgament bundle, do not start it
                if (bundle.getHeaders().get(Constants.FRAGMENT_HOST) == null) {
                    bundle.start();
                    Log.d(LOG_TAG, "Started bundle " + bundle.getSymbolicName());
                }
            } catch (BundleException ex) {
                Log.e(LOG_TAG, "Could not start bundle", ex);
            }

        }

    }

    @Override
    protected final void onStop() {
        super.onStop();
        try {
            framework.stop();
            framework.waitForStop(WAIT_TIMEOUT);
        } catch (Exception ex) {
            Log.e(LOG_TAG, "Could nost stop framework", ex);
        }
    }
}
